local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikUnitFrame = BaikCore.BaikUnitFrame
local Log = BaikCore.Log
local Hook = BaikCore.Hook
local LayoutAttrib = BaikCore.LayoutAttrib

-- Add Module
local TargetFrameMod = BaikUnitFrame:NewModule("TargetFrameMod", "AceHook-3.0")

-- UI Layout that doesn't get moved
local DYNAMIC_TABLE = {
    TargetFrame,
}

local STATIC_TABLE = {
    ----TARGET-----
    [TargetFrameTextureFrameName] = LayoutAttrib:Create(100, 12, "CENTER", -50, 36),
    -- Healthbar
    [TargetFrameHealthBar] = LayoutAttrib:Create(119, 19, "TOPRIGHT", -106, -22),
    [TargetFrameTextureFrameHealthBarText] = LayoutAttrib:Create(nil, nil, "CENTER", -50, 18),
    [TargetFrameTextureFrameHealthBarTextLeft] = LayoutAttrib:Create(nil, nil, "LEFT", 8, 18),
    [TargetFrameTextureFrameHealthBarTextRight] = LayoutAttrib:Create(nil, nil, "RIGHT", -110, 18),
    -- Manabr
    [TargetFrameManaBar] = LayoutAttrib:Create(119, 12, "TOPRIGHT", -106, -41),
    [TargetFrameTextureFrameManaBarText] = LayoutAttrib:Create(nil, nil, "CENTER", -50, 3),
    [TargetFrameTextureFrameManaBarTextLeft] = LayoutAttrib:Create(nil, nil, "LEFT", 8, 3),
    [TargetFrameTextureFrameManaBarTextRight] = LayoutAttrib:Create(nil, nil, "RIGHT", -110, 3),

    ----FOCUS-----
    [FocusFrameTextureFrameName] = LayoutAttrib:Create(100, 12, "CENTER", -50, 36),
    -- Healthbar
    [FocusFrameHealthBar] = LayoutAttrib:Create(119, 19, "TOPRIGHT", -106, -22),
    [FocusFrameTextureFrameHealthBarText] = LayoutAttrib:Create(nil, nil, "CENTER", -50, 18),
    [FocusFrameTextureFrameHealthBarTextLeft] = LayoutAttrib:Create(nil, nil, "LEFT", 8, 18),
    [FocusFrameTextureFrameHealthBarTextRight] = LayoutAttrib:Create(nil, nil, "RIGHT", -110, 18),
    -- Manabr
    [FocusFrameManaBar] = LayoutAttrib:Create(119, 12, "TOPRIGHT", -106, -41),
    [FocusFrameTextureFrameManaBarText] = LayoutAttrib:Create(nil, nil, "CENTER", -50, 3),
    [FocusFrameTextureFrameManaBarTextLeft] = LayoutAttrib:Create(nil, nil, "LEFT", 8, 3),
    [FocusFrameTextureFrameManaBarTextRight] = LayoutAttrib:Create(nil, nil, "RIGHT", -110, 3),
}

-- Private Methods
local function UnsetMinus(unitFrame)
    local classification = UnitClassification(unitFrame.unit)
    if classification == "minus" then
        Log:d("Minus detected")
        unitFrame.borderTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame")
        unitFrame.Background:SetSize(119,25);
        unitFrame.Background:SetPoint("BOTTOMLEFT", unitFrame, "BOTTOMLEFT", 7, 35);
        unitFrame.threatIndicator:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Flash")
        unitFrame.threatIndicator:SetTexCoord(0, 0.9453125, 0, 0.181640625);
        unitFrame.threatIndicator:SetWidth(242);
        unitFrame.threatIndicator:SetHeight(93);
        unitFrame.threatIndicator:SetPoint("TOPLEFT", unitFrame, "TOPLEFT", -24, 0)
    end
end

local function SetDynamic(unitFrame)
    unitFrame.nameBackground:SetVertexColor(0, 0, 0, 0.5)
    UnsetMinus(unitFrame)
end

local function LoadDynamic()
    Log:i("Syncing target dynamic frames")
    for _, frame in  pairs(DYNAMIC_TABLE) do
        SetDynamic(frame)
    end
end

local function LoadStatic()
    Log:i("Syncing target static frames")
    for frame, layout in pairs(STATIC_TABLE) do
        layout:SyncRegion(frame, true)
    end
end

-- Hooks
function TargetFrameMod:TargetFrame_CheckClassification(unitFrame, forceNormalTexture)
    SetDynamic(unitFrame)
end

-- Ace3 Callbacks
function TargetFrameMod:OnInitialize()
    Log:v("TargetFrameMod_OnInitialize")
end

function TargetFrameMod:OnEnable()
    Log:v("TargetFrameMod_OnEnable")
    LoadStatic()
    LoadDynamic()
    Hook:EnsureSecureHook(self, "TargetFrame_CheckClassification")
end

function TargetFrameMod:OnDisable()
    Log:v("TargetFrameMod_OnDisable")
    self:UnhookAll()
end
